# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import proxy
from openstack import resource
from openstack import utils

from otcextensions.sdk.auto_scaling.v1 import activity as _activity
from otcextensions.sdk.auto_scaling.v1 import config as _config
from otcextensions.sdk.auto_scaling.v1 import group as _group
from otcextensions.sdk.auto_scaling.v1 import instance as _instance
from otcextensions.sdk.auto_scaling.v1 import policy as _policy
from otcextensions.sdk.auto_scaling.v1 import quota as _quota


class Proxy(proxy.Proxy):

    skip_discovery = True

    # ======== Groups ========
    def groups(self, **query):
        """Retrieve a generator of groups

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * ``name``: group name
            * ``status``: group status, ``INSERVICE``, ``PAUSED``, ``ERROR``
            * ``scaling_configuration_id``: scaling configuration id
            * ``marker``:  pagination marker, known as ``start_number``
            * ``limit``: pagination limit

        :returns: A generator of group
            (:class:`~otcextensions.sdk.auto_scaling.v1.group.Group`) instances
        """
        return self._list(_group.Group, **query)

    def create_group(self, **attrs):
        """Create a new group from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`,
            comprised of the properties on the Group class.
        :returns: The results of group creation
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
        """
        return self._create(
            _group.Group, prepend_key=False, **attrs
        )

    def update_group(self, group, **attrs):
        """update group with attributes

        :param group: The value can be the ID of a group
            or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`,
            comprised of the properties on the Group class.
        :returns: The results of group creation
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
        """
        return self._update(
            _group.Group, group, prepend_key=False, **attrs)

    def get_group(self, group):
        """Get a group

        :param group: The value can be the ID of a group
             or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
             instance.
        :returns: Group instance
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
        """
        return self._get(
            _group.Group, group,
        )

    def delete_group(self, group, ignore_missing=True, force_delete=False):
        """Delete a group

        :param group: The value can be the ID of a group
             or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent group.
        :param bool force_delete: When set to ``True`` indicates to forcibly
            delete an AS group, remove the ECS instances and release them when
            the AS group is running instances or performing scaling actions.
        """
        res = self._get_resource(_group.Group, group)
        try:
            del_gr = res.delete(self, force_delete=force_delete)
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            raise
        return del_gr

    def find_group(self, name_or_id, ignore_missing=True):
        """Find a single group

        :param name_or_id: The name or ID of a group
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent group.

        :returns: ``None``
        """
        return self._find(
            _group.Group, name_or_id,
            ignore_missing=ignore_missing,
        )

    def resume_group(self, group):
        """resume group

        :param group: The value can be the ID of a group
             or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
             instance.
        """
        group = self._get_resource(
            _group.Group, group)
        return group.resume(self)

    def pause_group(self, group):
        """pause group

        :param group: The value can be the ID of a group
             or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
             instance.
        """
        group = self._get_resource(
            _group.Group, group
        )
        return group.pause(self)

    def wait_for_group(self, group, status='INSERVICE', failures=None,
                       interval=2, wait=180):
        """Wait for a group to be in a particular status.

        :param group: The value can be the ID of a group
            or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            instance
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        group = self._get_resource(
            _group.Group, group
        )
        failures = ['ERROR'] if failures is None else failures
        return resource.wait_for_status(
            self, group, status, failures, interval, wait
        )

    def wait_for_delete_group(self, group, interval=2, wait=60):
        """Wait for the group to be deleted.

        :param group:
            The :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            or group ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 60.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        group = self._get_resource(_group.Group, group)
        return resource.wait_for_delete(self, group, interval, wait)

    # ======== Configurations ========
    def configs(self, **query):
        """Retrieve a generator of configs

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * ``name``: configuration name
            * ``image_id``: image id
            * ``marker``:  pagination marker
            * ``limit``: pagination limit

        :returns: A generator of config
            (:class:`~otcextensions.sdk.auto_scaling.v1.config.Config`)
            instances
        """
        return self._list(_config.Config, **query)

    def create_config(self, name, **attrs):
        """Create a new config from config name and instance-config attributes

        :param name: auto scaling config name
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.auto_scaling.v1.config.InstanceConfig`
            , comprised of the properties on the InstanceConfig class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
        """
        return self._create(
            _config.Config,
            prepend_key=False,
            name=name,
            **attrs
        )

    def get_config(self, config):
        """Get a config

        :param config: The value can be the ID of a config
             or a :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
             instance.
        :returns: Config instance
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
        """
        return self._get(_config.Config, config)

    # Name is not unique, so find might return multiple results
    def find_config(self, name_or_id, ignore_missing=True):
        """Get a config

        :param name_or_id: The name or ID of a config
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the config does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent config.
        :returns: Config instance
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
        """
        return self._find(
            _config.Config, name_or_id,
            ignore_missing=ignore_missing,
        )

    def delete_config(self, config, ignore_missing=True):
        """Delete a config

        :param config: The value can be the ID of a config
            or a :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the config does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent config.

        :returns: Config been deleted
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
        """
        return self._delete(
            _config.Config, config,
            ignore_missing=ignore_missing,
        )

    def batch_delete_configs(self, configs):
        """batch delete configs

        :param configs: The list item value can be the ID of a config
            or a :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
            instance.
        """
        config = _config.Config()
        return config.batch_delete(self, configs)

    # ======== Policy ========
    def policies(self, group, **query):
        """Retrieve a generator of policies

        :param group: The value can be the ID of a group
            or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            instance.
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * ``name``: policy name
            * ``type``: policy type
            * ``scaling_group_id``: scaling group id the policy applied to
            * ``marker``:  pagination marker
            * ``limit``: pagination limit

        :returns: A generator of policy
            (:class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`)
            instances
        """
        group = self._get_resource(_group.Group, group)
        return self._list(
            _policy.Policy,
            base_path='/scaling_policy/{id}/list'.format(id=group.id), **query)

    def create_policy(self, **attrs):
        """Create a new policy from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`,
            comprised of the properties on the Policy class.
        :returns: The results of policy creation
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
        """
        return self._create(_policy.Policy, prepend_key=False, **attrs)

    def update_policy(self, policy, **attrs):
        """update policy with attributes

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`,
            comprised of the properties on the Policy class.
        :returns: The results of policy creation
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
        """
        return self._update(_policy.Policy, policy, prepend_key=False, **attrs)

    def get_policy(self, policy):
        """Get a policy

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        :returns: Policy instance
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
        """
        return self._get(_policy.Policy, policy)

    def delete_policy(self, policy, ignore_missing=True):
        """Delete a policy

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the policy does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent policy.

        :returns: Policy been deleted
        :rtype: :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
        """
        return self._delete(_policy.Policy, policy,
                            ignore_missing=ignore_missing)

    def find_policy(self, name_or_id, group, ignore_missing=True):
        """Find a single policy

        :param name_or_id: The name or ID of a policy
        :param group: ID of a group
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the policy does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent policy.

        :returns: ``None``
        """
        group = self._get_resource(_group.Group, group)
        return self._find(_policy.Policy, name_or_id,
                          ignore_missing=ignore_missing,
                          group_id=group.id)

    def execute_policy(self, policy):
        """execute policy

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        """
        policy = self._get_resource(_policy.Policy, policy)
        policy.execute(self)

    def resume_policy(self, policy):
        """resume policy

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        """
        policy = self._get_resource(_policy.Policy, policy)
        policy.resume(self)

    def pause_policy(self, policy):
        """pause policy

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.auto_scaling.v1.policy.Policy`
             instance.
        """
        policy = self._get_resource(_policy.Policy, policy)
        policy.pause(self)

    # ======== Instances ========
    def instances(self, group, **query):
        """Retrieve a generator of instances

        :param group: The value can be the ID of a group
             or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
             instance.
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * ``health_status``: instance health status
            * ``lifecycle_status``: policy type
            * ``scaling_group_id``: scaling group id the policy applied to
            * ``marker``:  pagination marker
            * ``limit``: pagination limit

        :returns: A generator of instances with type
            (:class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`)
        """
        group = self._get_resource(_group.Group, group)
        return self._list(
            _instance.Instance,
            base_path='/scaling_group_instance/{id}/list'.format(id=group.id),
            **query)

    def find_instance(self, name_or_id, group, ignore_missing=True):
        """Find a single instance

        :param name_or_id: The name or ID of a instance.
        :param group: ID of a group
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns:
            One :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            or None.
        """
        group = self._get_resource(_group.Group, group)
        return self._find(_instance.Instance, name_or_id,
                          ignore_missing=ignore_missing,
                          group_id=group.id)

    def remove_instance(self, instance, delete_instance=False,
                        ignore_missing=True):
        """Remove an instance of auto scaling group

        :precondition:
            * the instance must in ``INSERVICE`` status
            * after remove the instance number of auto scaling group should not
                be less than min instance number
            * The own auto scaling group should not in scaling status

        :param instance: The value can be the ID of a instance or a
            :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            instance.
        :param bool delete_instance: When set to ``True``, instance will be
            deleted after removed.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the config does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent config.

        :returns: None
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.remove(self,
                               delete_instance=delete_instance,
                               ignore_missing=ignore_missing)

    def batch_instance_action(
            self, group, instances,
            action, delete_instance=False):
        """Batch add instances for auto scaling group

        :param group: The group which instances will be added to,
            The value can be the ID of a group or a
            :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            instance.
        :param instances: The list item value can be ID of an instance or a
            :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            instance
        :param action: Action type
            [``ADD``, ``REMOVE``, ``PROTECT``, ``UNPROTECT``]
        :param delete_instance: When set to ``True``, instance will be
            deleted after removed
        """
        group = self._get_resource(_group.Group, group)
        instance = _instance.Instance(scaling_group_id=group.id)
        return instance.batch_action(self, instances, action, delete_instance)

    def wait_for_instance(self, instance, status='INSERVICE', failures=None,
                          interval=2, wait=180):
        """Wait for an instance to be in a particular status.

        :param instance:
            The :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            or instance ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        instance = self._get_resource(_instance.Instance, instance)
        failures = ['ERROR'] if failures is None else failures
        for count in utils.iterate_timeout(
            timeout=wait,
            message="Timeout waiting for instance to be in {status} status"
                    .format(status=status),
            wait=interval
        ):
            instance = self._find(_instance.Instance, name_or_id=instance.id,
                                  group_id=instance.scaling_group_id)
            if instance and instance.lifecycle_state == status:
                return instance

    def wait_for_delete_instance(self, instance, interval=2, wait=180):
        """Wait for the instance to be deleted.

        :param instance:
            The :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            or instance ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        instance = self._get_resource(_instance.Instance, instance)
        for count in utils.iterate_timeout(
            timeout=wait,
            message="Timeout waiting for instance to delete",
            wait=interval
        ):
            instance = self._find(_instance.Instance, name_or_id=instance.id,
                                  group_id=instance.scaling_group_id,
                                  ignore_missing=True)
            if instance is None:
                return

    # ======== Activities ========
    def activities(self, group, **query):
        """Retrieve a generator of Activity

        :param group: The value can be the ID of a group
            or a :class:`~otcextensions.sdk.auto_scaling.v1.group.Group`
            instance.
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * ``start_time``: activity start time
            * ``end_time``: activity end time
            * ``marker``:  pagination marker, known as ``start_number``
            * ``limit``: pagination limit

        :returns: A generator of group
            (:class:`~otcextensions.sdk.auto_scaling.v1.activity.Activity`)
            instances
        """
        group = self._get_resource(_group.Group, group)
        return self._list(_activity.Activity,
                          scaling_group_id=group.id,
                          **query)

    # ======== Quotas ========
    def quotas(self, group=None):
        """Retrieve a generator of Quota

        :param group: If group is set, will query quota for the group instead
            of quota of user. The value can be the ID of a group or a
            :class:`~otcextensions.sdk.auto_scaling.v1.group.Group` instance.
        :returns: A generator of quota
            (:class:`~otcextensions.sdk.auto_scaling.v1.quota.Quota`) instances
        """
        if group:
            group = self._get_resource(_group.Group, group)
            return self._list(_quota.ScalingQuota,
                              paginated=False,
                              scaling_group_id=group.id)
        else:
            return self._list(_quota.Quota, paginated=False)

    # ======== Project cleanup ========
    def _get_cleanup_dependencies(self):
        return {
            'auto_scaling': {
                'before': ['compute', 'block_storage']
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=None,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None,
                         skip_resources=None):
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.rds.v3 import _base


class Instance(_base.Resource):

    base_path = '/instances'
    resources_key = 'instances'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'type', 'datastore_type', 'router_id',
        'network_id', 'limit', 'offset',
        network_id='subnet_id',
        router_id='vpc_id')

    #: Availability Zone.
    #: *Type:string*
    availability_zone = resource.Body('availability_zone')
    # TODO(not_gtema): extract backup strategy into separate type
    #: Backup Strategy.
    #: *Type: dict*
    backup_strategy = resource.Body('backup_strategy', type=dict)
    #: Specifies the billing information, which is pay-per-use.
    #: *Type:dict*
    charge_info = resource.Body('charge_info', type=dict)
    #: Parameter configuration ID.
    #: *Type:uuid*
    configuration_id = resource.Body('configuration_id')
    #: Instance created time.
    #: *Type:string*
    created_at = resource.Body('created')
    #: Data store information.
    #: *Type:dict*
    datastore = resource.Body('datastore', type=dict)
    #: Datastore type information (for querying).
    #: *Type:string*
    datastore_type = resource.Body('datastore_type')
    #: Disk Encryption Key Id.
    #: *Type:uuid*
    disk_encryption_id = resource.Body('disk_encryption_id')
    #: Flavor ID
    #: *Type:uuid*
    flavor_ref = resource.Body('flavor_ref')
    #: Async job id
    #: *Type:uuid*
    job_id = resource.Body('job_id')
    #: HighAvailability configuration parameters.
    #: *Type:dict*
    ha = resource.Body('ha', type=dict)
    #: Maintenance time window.
    #: *Type:string*
    maintenance_window = resource.Body('maintenance_window')
    #: Node information
    #: Indicates the primary/standby DB instance information.
    #: *Type:list*
    nodes = resource.Body('nodes', type=list)
    #: Password of the default user.
    #: *Type:string*
    password = resource.Body('password')
    #: Database listen port number.
    #: *Type:int*
    port = resource.Body('port', type=int)
    #: Private IP address List.
    #: *Type:list*
    private_ips = resource.Body('private_ips', type=list)
    #: Public IP address List,
    #: *Type:list*
    public_ips = resource.Body('public_ips', type=list)
    #: Region where DB is deployed.
    #: *Type:string*
    region = resource.Body('region')
    #: list of associated DB instances.
    #: *Type:list*
    related_instances = resource.Body('related_instance', type=list)
    #: Specifies the DB instance ID, which is used to create a read replica.
    #: *Type:uuid*
    replica_of_id = resource.Body('replica_of_id')
    #: Specifies the restoration point for instance recovery.
    #: *Type:dict*
    restore_point = resource.Body('restore_point', type=dict)
    #: Recovery time period for instance.
    #: *Type:string*
    restore_time = resource.Body('restore_time')
    #: Neutron router ID.
    #: *Type:uuid*
    router_id = resource.Body('vpc_id')
    #: Security Group Id.
    #: *Type:uuid*
    security_group_id = resource.Body('security_group_id')
    #: Id of network.
    #: *Type:uuid*
    network_id = resource.Body('subnet_id')
    #: Instance status.
    #: *Type:string*
    status = resource.Body('status')
    #: Switch Strategy. The value can be reliability or availability,
    #: indicating the reliability first and availability first, respectively.
    #: *Type:string*
    switch_strategy = resource.Body('switch_strategy')
    #: Lists the tags and their values attached to the instance.
    #: *Type:dict*
    tags = resource.Body('tags', type=list)
    #: Time Zone.
    #: *Type:string*
    time_zone = resource.Body('time_zone')
    #: Instance type Single/Ha/Replica.,
    #: *Type:string*
    type = resource.Body('type')
    # datastore: Instance updated time.
    #: *Type:string*
    updated_at = resource.Body('updated')
    #: Default user of the DB
    #: *Type:string*
    user_name = resource.Body('db_user_name')
    #: Volume information
    #: *Type:dict*
    volume = resource.Body('volume', type=dict)

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response, error_message=error_message)
        if has_body:
            try:
                body = response.json()
                # Need to merge job_id on the root with other data inside of
                # 'instance' container
                job_id = body.get('job_id')
                if 'instance' in body:
                    body = body['instance']
                    if job_id:
                        body['job_id'] = job_id

                body_attrs = self._consume_body_attrs(body)

                if self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body)

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
            :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
            the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
            underlying methods, such as to
            :meth:`~openstack.resource.Resource.existing`
            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
            or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
            than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
            is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)

        result = cls._find(session, name_or_id, id=name_or_id, **params)
        if not result:
            result = cls._find(session, name_or_id, name=name_or_id, **params)

        if result:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    @classmethod
    def _find(cls, session, name_or_id, **params):
        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

    def _action(self, session, request_body):
        url = utils.urljoin(self.base_path, self.id, 'action')
        response = session.post(url, json=request_body)
        exceptions.raise_from_response(response)

    def _tag_action(self, session, request_body):
        url = utils.urljoin(self.base_path, self.id, 'tags', 'action')
        response = session.post(url, json=request_body)
        exceptions.raise_from_response(response)

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None, **params):
        """Get a remote resource based on this instance.

        :param session: The session to use for making this request.
            :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param boolean requires_id: A boolean indicating whether resource ID
            should be part of the requested URI.
        :param str base_path: Base part of the URI for fetching resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param str error_message: An Error message to be returned if
            requested object does not exist.
        :param dict params: Additional parameters that can be consumed.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_fetch` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
            the resource was not found.
        """
        data = self.list(session, paginated=False, id=self.id)
        result = self._get_one_match(self.id, data)
        if not result:
            raise exceptions.ResourceNotFound(
                "No Instance found for %s" % (self.id))

        self._body.attributes.update(result._body.attributes)
        self._body.clean()

        return self

    def fetch_restore_times(self, session):
        """List possible restore times for the instance.
        """
        url = utils.urljoin(self.base_path, self.id, 'restore-time')
        response = session.get(url)
        self._translate_response(response)
        return self.restore_time

    def get_instance_configuration(self, session):
        raise NotImplementedError()

    def update_instance_configuration(self, session, values):
        raise NotImplementedError()

    def restore(self, session, backup=None, restore_time=None):
        """Restore instance from the backup of PIR.
        """
        url = utils.urljoin(self.base_path, 'recovery')

        body = {
            'source': None,
            'target': {'instance_id': self.id}
        }

        if backup:
            body['source'] = {'type': 'backup', 'backup_id': backup.id}
        elif restore_time:
            body['source'] = {
                'type': 'timestamp',
                'restore_time': restore_time
            }

        body['source']['instance_id'] = self.id

        response = session.post(url, json=body)
        exceptions.raise_from_response(response)
        job_id = response.json().get('job_id')

        return job_id

    def get_backup_policy(self, session):
        """Get instance backup policy
        """
        url = utils.urljoin(self.base_path, self.id, 'backups', 'policy')
        response = session.get(url)
        exceptions.raise_from_response(response)

        return response.json().get('backup_policy')

    def set_backup_policy(self, session, keep_days, start_time=None,
                          period=None):
        """Set instance backup policy
        """
        url = utils.urljoin(self.base_path, self.id, 'backups', 'policy')
        body = {
            'keep_days': keep_days
        }
        if start_time:
            body['start_time'] = start_time
        if period:
            body['period'] = period
        response = session.put(url, json={'backup_policy': body})
        exceptions.raise_from_response(response)

        return None

    def get_logs(self, session, log_type, start_date, end_date,
                 offset, limit, level):
        """Get instance logs

        :param session: The session to use for making this request.
            :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str log_type: The type of logs to query:
            'errorlog' or 'slowlog'.
        :param str start_date: Start date of the of the log query. Format:
            %Y-%m-%dT%H:%M:%S%z where z is the tzinfo in HHMM format.
        :param str end_date: End date of the of the log query. Format:
            %Y-%m-%dT%H:%M:%S%z where z is the tzinfo in HHMM format.
        :param int offset: .
        :param int limit: Specifies the number of records on a page. Its value
            range is from 1 to 100.
        :param str level: Specifies the log level.

        """
        url_params = log_type + '?' + '&'.join([
            'start_date=' + start_date,
            'end_date=' + end_date,
            'offset=' + str(offset),
            'limit=' + str(limit),
            'level=' + level
        ])
        url = utils.urljoin(self.base_path, self.id, url_params)
        response = session.get(url)
        exceptions.raise_from_response(response)
        return response.json()

    def restart(self, session):
        """Restart the database instance
        """
        self._action(session, {"restart": {}})

    def enlarge_volume(self, session, size):
        """Enlarge the instance volume
        """
        self._action(session, {"enlarge_volume": {"size": int(size)}})

    def update_flavor(self, session, spec_code):
        """Chage the instance's flavor
        """
        self._action(session, {"resize_flavor": {"spec_code": spec_code}})

    def add_tag(self, session, key, value):
        """Add tag to instance
        """
        request_body = {
            "action": "create",
            "tags": [{
                "key": key,
                "value": value
            }]
        }
        self._tag_action(session, request_body)

    def remove_tag(self, session, key):
        """Remove tag from instance
        """
        request_body = {
            "action": "delete",
            "tags": [{
                "key": key
            }]
        }
        self._tag_action(session, request_body)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Cluster Certificates v2 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_cluster_certificate(obj):
    """Flatten the structure of the cluster node into a single dict
    """
    data = {
        'name': obj.context.get('name', None),
        'cluster': obj.context.get('cluster', None),
        'user': obj.context.get('user', None),
        'ca': obj.ca,
        'client_certificate': obj.client_certificate,
        'client_key': obj.client_key,
    }

    return data


class ShowCCEClusterCertificates(command.ShowOne):
    _description = _('Show Cluster certificates details')
    columns = (
        'name',
        'cluster',
        'user',
        'ca',
        'client_certificate',
        'client_key')

    def get_parser(self, prog_name):
        parser = super(ShowCCEClusterCertificates, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        cluster = client.find_cluster(parsed_args.cluster,
                                      ignore_missing=False)

        obj = client.get_cluster_certificates(
            cluster=cluster.id
        )

        flat_data = _flatten_cluster_certificate(obj)
        columns = tuple(flat_data)

        data = utils.get_dict_properties(
            flat_data,
            columns)

        return columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest.mock import MagicMock

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.ak_auth import AKRequestsAuth
from otcextensions.sdk.obs.v1 import _proxy
from otcextensions.sdk.obs.v1 import container as _container
from otcextensions.sdk.obs.v1 import obj as _obj


class TestObsProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestObsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

        self._ak_auth = AKRequestsAuth(
            access_key='ak',
            secret_access_key='sk',
            host='host',
            region='regio',
            service='OBS')
        self.proxy._ak_auth = self._ak_auth
        self.proxy.region_name = 'regio'
        self.proxy.get_container_endpoint = MagicMock(
            return_value='https://container.obs.regio.otc.t-systems.com'
        )

    def test_containers(self):
        self.verify_list(
            self.proxy.containers, _container.Container,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            expected_kwargs={
                'requests_auth': self._ak_auth
            }
        )

    def test_create_container(self):
        self.verify_create(
            self.proxy.create_container, _container.Container,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
            method_kwargs={
                'name': 'container'
            },
            expected_kwargs={
                'name': 'container',
                'endpoint_override':
                    'https://container.obs.regio.otc.t-systems.com',
                'requests_auth': self._ak_auth
            }
        )

    def test_delete_container(self):
        self.verify_delete(
            self.proxy.delete_container,
            _container.Container,
            ignore_missing=True,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
            expected_kwargs={
                'endpoint_override': 'https://container.obs.regio.'
                                     'otc.t-systems.com',
                'ignore_missing': True,
                'requests_auth': self._ak_auth
            }
        )

    def test_get_container_metadata(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.get_container_metadata,
            'container'
        )

    def test_set_container_metadata(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.set_container_metadata,
            'container',
            metadata={}
        )

    def test_delete_container_metadata(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.delete_container_metadata,
            'container',
            keys={}
        )

    def test_objects(self):
        self.verify_list(
            self.proxy.objects, _obj.Object,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            method_args={
                'container': 'container'
            },
            expected_kwargs={
                'endpoint_override': 'https://container.obs.regio.'
                                     'otc.t-systems.com',
                'requests_auth': self._ak_auth
            },
            expected_args=[]
        )

    def test_create_object(self):
        self.verify_create(
            self.proxy.create_object, _obj.Object,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
            method_kwargs={
                'name': 'nm',
                'data': 'test',
                'container': 'container'
            },
            expected_kwargs={
                'container': 'container',
                'data': 'test',
                'name': 'nm',
                'endpoint_override': 'https://container.obs.regio.otc.'
                                     't-systems.com',
                'requests_auth': self._ak_auth
            }
        )

    def test_delete_object(self):
        self.verify_delete(
            self.proxy.delete_object, _obj.Object,
            ignore_missing=True,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
            method_kwargs={'container': 'container'},
            expected_kwargs={
                'endpoint_override': 'https://container.obs.regio.'
                                     'otc.t-systems.com',
                'ignore_missing': True,
                'container': 'container',
                'requests_auth': self._ak_auth
            }
        )

    def test_download_object(self):
        self._verify(
            'otcextensions.sdk.obs.v1.obj.Object.download',
            self.proxy.download_object,
            method_args=[{}],
            method_kwargs={'container': 'container'},
            expected_args=[self.proxy],
            expected_kwargs={
                'filename': '-',
                'endpoint_override': 'https://container.obs.regio.'
                                     'otc.t-systems.com',
                'requests_auth': self._ak_auth
            }
        )

    def test_stream_object(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.stream_object,
            'container',
        )

    def test_copy_object(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.copy_object,
        )

    def test_get_object_metadata(self):
        self._verify(
            "otcextensions.sdk.sdk_proxy.Proxy._head",
            self.proxy.get_object_metadata,
            method_args=['object'],
            method_kwargs={'container': 'container'},
            expected_args=[_obj.Object, 'object'],
            expected_kwargs={
                'container': 'container',
                'endpoint_override': 'https://container.obs.regio.'
                                     'otc.t-systems.com',
                'requests_auth': self._ak_auth
            })

    def test_set_object_metadata(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.set_object_metadata,
            'container',
        )

    def test_delete_object_metadata(self):
        self.assertRaises(
            NotImplementedError,
            self.proxy.delete_object_metadata,
            'container',
        )


class TestExtractName(TestObsProxy):

    def test_extract_name(self):

        self.assertEqual(
            ['bucket'],
            self.proxy._extract_name('/', project_id='123')
        )
        self.assertEqual(
            ['object'],
            self.proxy._extract_name('/dummy', project_id='123')
        )
        self.assertEqual(
            ['object'],
            self.proxy._extract_name('/dummy/dummy2', project_id='123')
        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create a DNS zone
"""
import openstack
from otcextensions.sdk.dns.v2.zone import Router

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


zone = conn.dns.create_zone(
    name='private-zone',
    description='My private Zone',
    zone_type='private',
    ttl=300,
    email='zone-admin@example.com',
    router=Router(router_id='vpc_id',
                  router_region='eu-de'
                  )
)
print(zone)

'''
attrs = {
    "name": "private-zone.",
    "description": "My private Zone",
    "zone_type": "private",
    "email": "zone-admin@example.com",
    "router": {
        "router_id": "vpc_id",
        "router_region": "eu-de"
    }
}

zone = conn.dns.create_zone(**attrs)
print(zone)
'''



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class AvailabilityZone(resource.Resource):

    resources_key = 'available_zones'

    base_path = '/availableZones'

    # capabilities
    allow_list = True

    #: Properties
    #: Available Zone ID.
    id = resource.Body('id')
    #: Available Zone code.
    code = resource.Body('code')
    #: Available Zone name.
    name = resource.Body('name')
    #: Port number of the Available Zone.
    port = resource.Body('port')
    #: An indicator of whether there are available resources in the
    # Available Zone
    resource_availability = resource.Body('resource_availability')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.tests.unit import base
from otcextensions.sdk.dws.v1 import snapshot

EXAMPLE = {
    'id': uuid.uuid4().hex,
    'name': 'snapshot-1',
    'description': 'snapshot description',
    'started': '2016-08-23T03:59:23Z',
    'finished': '2016-08-23T04:01:40Z',
    'size': 500,
    'status': 'AVAILABLE',
    'type': 'MANUAL',
    'cluster_id': uuid.uuid4().hex,
}

RESTORED_CLUSTER = {'id': uuid.uuid4().hex}


class TestSnapshot(base.TestCase):

    def setUp(self):
        super(TestSnapshot, self).setUp()

    def test_basic(self):
        sot = snapshot.Snapshot()

        self.assertEqual('/snapshots', sot.base_path)
        self.assertEqual('snapshots', sot.resources_key)
        self.assertEqual('snapshot', sot.resource_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)

    def test_make_it(self):
        sot = snapshot.Snapshot(**EXAMPLE)

        updated_sot_attrs = (
            'started',
            'finished',
        )
        self.assertEqual(EXAMPLE['started'], sot.created_at)
        self.assertEqual(EXAMPLE['finished'], sot.updated_at)

        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)


class TestRestore(base.TestCase):

    def setUp(self):
        super(TestRestore, self).setUp()

    def test_basic(self):
        sot = snapshot.Restore()

        self.assertEqual('/snapshots/%(snapshot_id)s/actions', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)

    def test_make_it(self):
        sot = snapshot.Restore(**RESTORED_CLUSTER)

        self.assertEqual(RESTORED_CLUSTER['id'], sot.id)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Distributed Cache Service Instance
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


instance = conn.dcs.create_instance(
    name="dcs-test",
    available_zones=["eu-de-03"],
    capacity=2,
    engine="Redis",
    engine_version="3.0.7",
    maintain_begin="02:00:00",
    maintain_end="06:00:00",
    password="Password.123",
    product_id="OTC_DCS_SINGLE",
    resource_spec_code="dcs.single_node",
    security_group_id="bb0e60ab-b6e0-4c61-a503-63213c18effa",
    subnet_id="25d24fc8-d019-4a34-9fff-0a09fde6a9cb",
    vpc_id="26ca2783-dc40-4e3a-95b1-5a0756441e12",
)
print(instance)

'''
attrs = {
    "name": "dcs-test",
    "engine": "Redis",
    "capacity": 2,
    "resource_spec_code": "dcs.single_node",
    "engine_version": "3.0.7",
    "vpc_id": "26ca2783-dc40-4e3a-95b1-5a0756441e12",
    "product_id": "OTC_DCS_SINGLE",
    "password": "Password.123",
    "user_id": "18569c6d589c4be3a300b6401c74d936",
    "user_name": "iam_user",
    "maintain_begin": "02:00:00",
    "maintain_end": "06:00:00",
    "enable_publicip": False,
    "enable_ssl": False,
    "service_upgrade": False,
    "service_task_id": "",
    "available_zones": [
        "eu-de-03"
    ],
    "subnet_id": "25d24fc8-d019-4a34-9fff-0a09fde6a9cb",
    "security_group_id": "bb0e60ab-b6e0-4c61-a503-63213c18effa"
}

instance = conn.dcs.create_instance(**attrs)
print(instance)
'''



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.smn.v2 import sms


EXAMPLE = {
    "message_id": "bf94b63a5dfb475994d3ac34664e24f2",
    "request_id": "9974c07f6d554a6d827956acbeb4be5f"
}


class TestSms(base.TestCase):

    def test_basic(self):
        sot = sms.Sms()
        path = '/notifications/sms'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = sms.Sms(**EXAMPLE)
        self.assertEqual(EXAMPLE['message_id'], sot.message_id)
        self.assertEqual(EXAMPLE['request_id'], sot.request_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import logging

from osc_lib import utils
from otcextensions import sdk
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

DEFAULT_API_VERSION = "1"
API_VERSION_OPTION = "os_modelartsv1_api_version"
API_NAME = "modelartsv1"
API_VERSIONS = {
    "1": "openstack.connection.Connection",
}


def make_client(instance):
    """Returns a modelartsv1 proxy"""

    conn = instance.sdk_connection

    if getattr(conn, "modelartsv1", None) is None:
        LOG.debug("OTC extensions are not registered. Do that now")
        sdk.register_otc_extensions(conn)

    LOG.debug(
        "ModelArtsv1 client initialized using OpenStack OTC SDK: %s",
        conn.modelartsv1,
    )
    return conn.modelartsv1


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        "--os-modelartsv1-api-version",
        metavar="<modelartsv1-api-version>",
        default=utils.env("OS_MODELARTSV1_API_VERSION"),
        help=_(
            "MODELARTSV1 API version, default=%s "
            "(Env: OS_MODELARTSV1_API_VERSION)"
        )
        % DEFAULT_API_VERSION,
    )
    return parser



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from otcextensions.sdk.modelartsv2.v2.dataset import DataSourceSpec
from otcextensions.sdk.modelartsv2.v2.dataset import LabelFormatSpec
from otcextensions.sdk.modelartsv2.v2.dataset import LabelSpec


class FileCopyProgressSpec(resource.Resource):
    #: Number of files that have been transferred.
    file_num_finished = resource.Body("file_num_finished", type=int)
    #: Total number of files.
    file_num_total = resource.Body("file_num_total", type=int)
    #: Size of the file that has been transferred, in bytes.
    file_size_finished = resource.Body("file_size_finished", type=int)
    #: Total file size, in bytes.
    file_size_total = resource.Body("file_size_total", type=int)


class DatasetImportTask(resource.Resource):
    base_path = "/datasets/%(uri_dataset_id)s/import-tasks"

    resources_key = "import_tasks"

    _query_mapping = resource.QueryParameters(
        "limit",
        "offset",
    )

    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_patch = True

    # Properties
    #: Number of labeled samples.
    annotated_sample_count = resource.Body("annotated_sample_count", type=int)
    #: Format of the labeling information.
    annotation_format = resource.Body("annotation_format")
    #: Time when a task is created.
    created_at = resource.Body("create_time", type=int)
    #: Data source.
    data_source = resource.Body("data_source", type=DataSourceSpec)
    #: Dataset ID.
    dataset_id = resource.Body("dataset_id")
    #: Dataset ID.
    uri_dataset_id = resource.URI("uri_dataset_id")
    #: Whether to import only hard examples.
    difficult_only = resource.Body("difficult_only", type=bool)
    #: Task running time, in seconds.
    elapsed_time = resource.Body("elapsed_time", type=int)
    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Do not import samples containing the specified label.
    excluded_labels = resource.Body(
        "excluded_labels", type=list, list_type=LabelSpec
    )
    #: Progress of file copy.
    file_statistics = resource.Body(
        "file_statistics", type=FileCopyProgressSpec
    )
    #: Whether to import data to the final state.
    final_annotation = resource.Body("final_annotation", type=bool)
    #: Number of files that have been transferred.
    finished_file_count = resource.Body("finished_file_count", type=int)
    #: Size of the file that has been transferred, in bytes.
    finished_file_size = resource.Body("finished_file_size", type=int)
    #: Whether to import labels.
    import_annotations = resource.Body("import_annotations", type=bool)
    #: Name of the subdirectory in the dataset storage directory after import.
    import_folder = resource.Body("import_folder")
    #: Data source.
    import_origin = resource.Body("import_origin")
    #: OBS path or manifest path to be imported.
    import_path = resource.Body("import_path")
    #: Import mode.
    import_type = resource.Body("import_type", type=int)
    #: Whether to import samples.
    import_samples = resource.Body("import_samples", type=bool)
    #: Number of imported samples.
    imported_sample_count = resource.Body("imported_sample_count", type=int)
    #: Number of imported subsamples.
    imported_sub_sample_count = resource.Body(
        "imported_sub_sample_count", type=int
    )
    #: Import samples containing the specified label.
    included_labels = resource.Body(
        "included_labels", type=list, list_type=LabelSpec
    )
    #: Label format.
    label_format = resource.Body("label_format", type=LabelFormatSpec)
    #: ID of a preprocessing task.
    processor_task_id = resource.Body("processor_task_id")
    #: Status of a preprocessing task.
    processor_task_status = resource.Body("processor_task_status", type=int)
    #: Status of an import task.
    status = resource.Body("status")
    #: ID of an import task.
    task_id = resource.Body("task_id", alternate_id=True)
    #: Total number of files.
    total_file_count = resource.Body("total_file_count", type=int)
    #: Total file size, in bytes.
    total_file_size = resource.Body("total_file_size", type=int)
    #: Total number of samples.
    total_sample_count = resource.Body("total_sample_count", type=int)
    #: Total number of subsamples generated from the parent samples.
    total_sub_sample_count = resource.Body("total_sub_sample_count", type=int)
    #: Number of samples to be confirmed.
    unconfirmed_sample_count = resource.Body(
        "unconfirmed_sample_count", type=int
    )
    #: Time when a task is updated.
    updated_at = resource.Body("update_ms", type=int)
    #: Whether the first row in the file is a column name.
    with_column_header = resource.Body("with_column_header", type=bool)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from keystoneauth1 import exceptions as _exceptions

import openstack.config
from openstack import connection
from openstack.tests import base

from otcextensions import sdk


#: Defines the OpenStack Client Config (OCC) cloud key in your OCC config
#: file, typically in $HOME/.config/openstack/clouds.yaml. That configuration
#: will determine where the functional tests will be run and what resource
#: defaults will be used to run the functional tests.
TEST_CLOUD_NAME = os.getenv('OS_CLOUD', 'otc')
TEST_CLOUD_REGION = openstack.config.get_cloud_region(cloud=TEST_CLOUD_NAME)


def _get_resource_value(resource_key, default):
    try:
        return TEST_CLOUD_REGION.config['functional'][resource_key]
    except KeyError:
        return default


IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.5-x86_64-disk')
FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')


class BaseFunctionalTest(base.TestCase):

    def setUp(self):
        super(BaseFunctionalTest, self).setUp()
        self.conn = connection.Connection(config=TEST_CLOUD_REGION)
        sdk.register_otc_extensions(self.conn)

    def addEmptyCleanup(self, func, *args, **kwargs):
        def cleanup():
            result = func(*args, **kwargs)
            self.assertIsNone(result)
        self.addCleanup(cleanup)

    # TODO(shade) Replace this with call to conn.has_service when we've merged
    #             the shade methods into Connection.
    def require_service(self, service_type, **kwargs):
        """Method to check whether a service exists

        Usage:
        class TestMeter(base.BaseFunctionalTest):
            ...
            def setUp(self):
                super(TestMeter, self).setUp()
                self.require_service('metering')

        :returns: True if the service exists, otherwise False.
        """
        try:
            self.conn.session.get_endpoint(service_type=service_type, **kwargs)
        except _exceptions.EndpointNotFound:
            self.skipTest('Service {service_type} not found in cloud'.format(
                service_type=service_type))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource


class Stream(resource.Resource):
    resource_key = 'log_streams'
    resources_key = 'log_streams'
    base_path = '/groups/%(log_group_id)s/streams'

    # capabilities
    allow_create = True
    allow_fetch = False
    allow_commit = False
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
    )

    # Properties
    #: Name of the log stream.
    name = resource.Body('log_stream_name')
    #: ID of the log stream.
    id = resource.Body('log_stream_id', alternate_id=True)
    #: Time when a log stream was created
    creation_time = resource.Body('creation_time')
    #: ID of the log group to which the log stream to be created will belong.
    log_group_id = resource.Body('log_group_id')
    #: Number of filters.
    filter_count = resource.Body('filter_count', type=int)
    #: Log stream tag.
    tag = resource.Body('tag')

    @classmethod
    def list(
        cls,
        session,
        paginated=True,
        base_path=None,
        allow_unknown_params=False,
        *,
        microversion=None,
        **params,
    ):
        session = cls._get_session(session)

        if microversion is None:
            microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path

        api_filters = cls._query_mapping._validate(
            params,
            base_path=base_path,
            allow_unknown_params=True,
        )
        query_params = cls._query_mapping._transpose(api_filters, cls)
        uri = base_path % params
        uri_params = {}

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion,
            )
            exceptions.raise_from_response(response)
            data = response.json()

            query_params.pop('marker', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)
                # We want that URI props are available on the resource
                raw_resource.update(uri_params)

                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource,
                )
                yield value

            return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.nat.v2 import gateway
from otcextensions.tests.unit.osclient.nat.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListNatGateways(fakes.TestNat):

    objects = fakes.FakeNatGateway.create_multiple(3)

    column_list_headers = ('Id', 'Name', 'Spec', 'Router Id', 'Status')

    columns = ('id', 'name', 'spec', 'router_id', 'status')

    data = []

    for s in objects:
        data.append(
            (s.id, s.name, s.spec, s.router_id, s.status))

    def setUp(self):
        super(TestListNatGateways, self).setUp()

        self.cmd = gateway.ListNatGateways(self.app, None)

        self.client.gateways = mock.Mock()
        self.client.api_mock = self.client.gateways

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--id', '2',
            '--name', '3',
            '--project-id', '4',
            '--spec', '5',
            '--router-id', '6',
            '--internal-network-id', '7',
            '--admin-state-up', '8',
            '--created-at', '9',
            '--status', '10'
        ]

        verifylist = [
            ('limit', 1),
            ('id', '2'),
            ('name', '3'),
            ('project_id', '4'),
            ('spec', '5'),
            ('router_id', '6'),
            ('internal_network_id', '7'),
            ('admin_state_up', '8'),
            ('created_at', '9'),
            ('status', '10'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            id='2',
            name='3',
            project_id='4',
            spec='5',
            router_id='6',
            internal_network_id='7',
            admin_state_up='8',
            created_at='9',
            status='10',
        )


class TestCreateNatGateway(fakes.TestNat):

    _data = fakes.FakeNatGateway.create_one()

    columns = (
        'admin_state_up',
        'created_at',
        'description',
        'id',
        'internal_network_id',
        'name',
        'project_id',
        'router_id',
        'spec',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateNatGateway, self).setUp()

        self.cmd = gateway.CreateNatGateway(self.app, None)

        self.client.create_gateway = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'test-gateway',
            '--router-id', 'test-router-uuid',
            '--internal-network-id', 'test-network-uuid',
            '--spec', '1',
        ]
        verifylist = [
            ('name', 'test-gateway'),
            ('router_id', 'test-router-uuid'),
            ('internal_network_id', 'test-network-uuid'),
            ('spec', '1'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_gateway.assert_called_with(
            name='test-gateway',
            router_id='test-router-uuid',
            internal_network_id='test-network-uuid',
            spec='1'
        )
        self.assertEqual(self.columns, columns)


class TestUpdateNatGateway(fakes.TestNat):

    _data = fakes.FakeNatGateway.create_one()

    columns = (
        'admin_state_up',
        'created_at',
        'description',
        'id',
        'internal_network_id',
        'name',
        'project_id',
        'router_id',
        'spec',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateNatGateway, self).setUp()

        self.cmd = gateway.UpdateNatGateway(self.app, None)

        self.client.find_gateway = mock.Mock(return_value=self._data)
        self.client.update_gateway = mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            self._data.name,
            '--name', 'test-gateway-updated',
            '--description', 'nat gateway updated',
            '--spec', '2',
        ]
        verifylist = [
            ('gateway', self._data.name),
            ('name', 'test-gateway-updated'),
            ('description', 'nat gateway updated'),
            ('spec', '2'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_gateway.assert_called_with(self._data.name)
        self.client.update_gateway.assert_called_with(
            self._data.id,
            name='test-gateway-updated',
            description='nat gateway updated',
            spec='2'
        )
        self.assertEqual(self.columns, columns)


class TestShowNatGateway(fakes.TestNat):

    _data = fakes.FakeNatGateway.create_one()

    columns = (
        'admin_state_up',
        'created_at',
        'description',
        'id',
        'internal_network_id',
        'name',
        'project_id',
        'router_id',
        'spec',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowNatGateway, self).setUp()

        self.cmd = gateway.ShowNatGateway(self.app, None)

        self.client.find_gateway = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('gateway', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_gateway.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_nat_gateway',
        ]

        verifylist = [
            ('gateway', 'unexist_nat_gateway'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_gateway = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_gateway.assert_called_with('unexist_nat_gateway')


class TestDeleteNatGateway(fakes.TestNat):

    _data = fakes.FakeNatGateway.create_multiple(2)

    def setUp(self):
        super(TestDeleteNatGateway, self).setUp()

        self.client.delete_gateway = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = gateway.DeleteNatGateway(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('gateway', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_gateway = (
            mock.Mock(return_value=self._data[0])
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_gateway.assert_called_with(self._data[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for nat_gw in self._data:
            arglist.append(nat_gw.name)

        verifylist = [
            ('gateway', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = self._data
        self.client.find_gateway = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for nat_gw in self._data:
            calls.append(call(nat_gw.id))
        self.client.delete_gateway.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_nat_gateway',
        ]
        verifylist = [
            ('gateway', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self._data[0], exceptions.CommandError]
        self.client.find_gateway = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 NAT Gateway(s) failed to delete.', str(e))

        self.client.find_gateway.assert_any_call(self._data[0].name)
        self.client.find_gateway.assert_any_call('unexist_nat_gateway')
        self.client.delete_gateway.assert_called_once_with(self._data[0].id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils as tests_utils

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import builtin_model
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestListBuiltInModels(fakes.TestModelartsv1):
    objects = fakes.FakeBuiltInModel.create_multiple(3)

    column_list_headers = (
        "Model Id",
        "Model name",
        "Model Usage",
        "Model Precision",
        "Model Size",
        "Created At",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.model_id,
                s.model_name,
                s.model_usage,
                s.model_precision,
                s.model_size,
                cli_utils.UnixTimestampFormatter(s.created_at),
            )
        )

    def setUp(self):
        super(TestListBuiltInModels, self).setUp()

        self.cmd = builtin_model.ListBuiltInModels(self.app, None)

        self.client.builtin_models = mock.Mock()
        self.client.api_mock = self.client.builtin_models

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--limit",
            "1",
            "--offset",
            "2",
            "--sort-by",
            "create_time",
            "--order",
            "asc",
            "--search-content",
            "test-model-name",
        ]

        verifylist = [
            ("limit", 1),
            ("offset", 2),
            ("sort_by", "create_time"),
            ("order", "asc"),
            ("search_content", "test-model-name"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            offset=2,
            sort_by="create_time",
            order="asc",
            search_content="test-model-name",
        )


class TestShowBuiltInModel(fakes.TestModelartsv1):
    _model = fakes.FakeBuiltInModel.create_one()
    columns = (
        "created_at",
        "engine_id",
        "engine_name",
        "engine_version",
        "model_dataset_format",
        "model_description_url",
        "model_id",
        "model_name",
        "model_precision",
        "model_size",
        "model_train_dataset",
        "model_usage",
        "parameter",
    )
    formatters = {
        "created_at": cli_utils.UnixTimestampFormatter,
        "parameter": cli_utils.YamlFormat,
    }
    data = fakes.gen_data(_model, columns, formatters=formatters)

    def setUp(self):
        super(TestShowBuiltInModel, self).setUp()

        self.cmd = builtin_model.ShowBuiltInModel(self.app, None)

        self.client.find_builtin_model = mock.Mock(return_value=self._model)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._model.model_name,
        ]

        verifylist = [("name", self._model.model_name)]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_builtin_model.assert_called_with(
            self._model.model_name, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import health_monitor

EXAMPLE = {
    'delay': 5,
    'domain_name': 'domain-name',
    'expected_codes': 'expected-codes',
    'http_method': 'http_method',
    'admin_state_up': True,
    'max_retries': 5,
    'max_retries_down': 5,
    'monitor_port': 80,
    'pool_id': 'pool-id',
    'pools': [{'id': 'uuid1'}, {'id': 'uuid2'}],
    'project_id': 'project-id',
    'timeout': 20,
    'type': 'type',
    'url_path': 'url-path',
    'created_at': 'created-at',
    'updated_at': 'updated-at',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = health_monitor.HealthMonitor()
        path = '/elb/healthmonitors'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = health_monitor.HealthMonitor(**EXAMPLE)
        self.assertEqual(EXAMPLE['delay'], sot.delay)
        self.assertEqual(EXAMPLE['domain_name'], sot.domain_name)
        self.assertEqual(EXAMPLE['expected_codes'], sot.expected_codes)
        self.assertEqual(EXAMPLE['http_method'], sot.http_method)
        self.assertEqual(EXAMPLE['admin_state_up'], sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['max_retries'], sot.max_retries)
        self.assertEqual(EXAMPLE['max_retries_down'], sot.max_retries_down)
        self.assertEqual(EXAMPLE['monitor_port'], sot.monitor_port)
        self.assertEqual(EXAMPLE['pool_id'], sot.pool_id)
        self.assertEqual(EXAMPLE['pools'], sot.pools)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['timeout'], sot.timeout)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['url_path'], sot.url_path)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack.dns.v2 import zone
from openstack.network.v2 import router
from openstack import utils
from otcextensions.sdk import dnsmixin


class Router(router.Router):
    """DNS Private Zone Router Resource"""

    router_id = resource.Body('router_id')
    router_region = resource.Body('router_region')


class Zone(dnsmixin.DNSProxyMixin, zone.Zone):
    """DNS ZONE Resource"""

    _query_mapping = resource.QueryParameters(
        'zone_type', 'limit', 'marker', 'offset', 'tags',
        zone_type='type')

    #: Recordset number of the zone
    record_num = resource.Body('record_num', type=int)
    #: A dictionary represent Router(VPC), for private zone
    router = resource.Body('router', type=Router)
    #: Router list associated to this zone
    routers = resource.Body('routers', type=list, list_type=Router)
    #: Zone type, if private, domain will only available in a special VPC.
    #: Valid values include `private`, `public`
    #: *Type: str*
    zone_type = resource.Body('zone_type')

    def _action(self, session, action, body):
        """Preform actions given the message body.

        """
        url = utils.urljoin(self.base_path, self.id, action)
        return session.post(
            url,
            json=body)

    def associate_router(self, session, **router):
        body = {'router': {}}
        body['router']['router_id'] = router.get('router_id')
        if 'router_region' in router:
            body['router']['router_region'] = router.get('router_region')
        return self._action(session, 'associaterouter', body)

    def disassociate_router(self, session, **router):
        body = {'router': {}}
        body['router']['router_id'] = router.get('router_id')
        if 'router_region' in router:
            body['router']['router_region'] = router.get('router_region')
        return self._action(session, 'disassociaterouter', body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional.sdk.auto_scaling.v1.base import TestAs

_logger = _log.setup_logging('openstack')


class TestASWorkflow(TestAs):

    def setUp(self):
        super(TestASWorkflow, self).setUp()
        self.create_network()
        self.initialize_as_group_with_instance()

    def test_01_find_as_group_by_name(self):
        as_group = self.client.find_group(
            name_or_id=TestAs.AS_GROUP_NAME)
        self.assertIsNotNone(as_group)
        self.assertEqual(as_group.name, self.AS_GROUP_NAME)

    def test_02_list_as_groups(self):
        as_group_list = list(self.client.groups())
        self.assertIsNotNone(as_group_list)

    def test_03_update_as_group(self):
        new_name = self.AS_GROUP_NAME
        as_group = self.client.update_group(
            group=TestAs.AS_GROUP, name=new_name,
            desire_instance_number=0)
        self.assertIsNotNone(as_group)
        self.assertEqual(as_group.name, new_name)

    def test_04_find_as_group_by_id(self):
        find_as_group = self.conn.auto_scaling.find_group(
            name_or_id=TestAs.AS_GROUP.id
        )
        self.assertEqual(TestAs.AS_GROUP.id, find_as_group.id)
        self.assertEqual(TestAs.AS_GROUP.name, find_as_group.name)

        self.deinitialize_as_group_with_instance()
        as_group = self.client.find_group(
            name_or_id=find_as_group.name)
        self.assertIsNone(as_group)

        self.destroy_network(TestAs.NETWORK, TestAs.KP_NAME)
        vpc = self.conn.vpc.find_vpc(
            TestAs.NETWORK.get('router_id'),
            ignore_missing=True)
        self.assertIsNone(vpc)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.sdk.deh.v1 import _proxy
from otcextensions.sdk.deh.v1 import host as _host
from otcextensions.sdk.deh.v1 import host_type as _host_type
from otcextensions.sdk.deh.v1 import server as _server

from openstack.tests.unit import test_proxy_base


class TestDehProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestDehProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_hosts(self):
        self.verify_list(
            self.proxy.hosts, _host.Host,
            expected_kwargs={
            }
        )

    def test_get_host(self):
        self.verify_get(
            self.proxy.get_host, _host.Host,
        )

    def test_find_host(self):
        self.verify_find(
            self.proxy.find_host, _host.Host,
        )

    def test_create_host(self):
        self.verify_create(
            self.proxy.create_host, _host.Host,
            method_kwargs={
                'x': 1,
                'y': '2'
            },
            expected_kwargs={
                'prepend_key': False,
                'x': 1,
                'y': '2'
            }
        )

    def test_delete_host(self):
        self.verify_delete(
            self.proxy.delete_host,
            _host.Host,
            ignore_missing=True,
        )

    def test_update_host(self):
        self.verify_update(
            self.proxy.update_host, _host.Host,
        )

    def test_servers(self):
        self.verify_list(
            self.proxy.servers, _server.Server,
            method_args=[{'id': 'host_id'}],
            method_kwargs={},
            expected_kwargs={
                'dedicated_host_id': 'host_id'
            },
            expected_args=[]
        )

    def test_host_types(self):
        self.verify_list(
            self.proxy.host_types, _host_type.HostType,
            method_args=['az'],
            method_kwargs={},
            expected_kwargs={
                'paginated': False,
                'availability_zone': 'az'
            },
            expected_args=[]
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Policy CLI implementation'''
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_policy(obj):
    """Flatten the structure of the node pool into a single dict
    """
    od = obj.operation_definition
    data = {
        'id': obj.id,
        'name': obj.name,
        'enabled': obj.enabled,
        'operation_type': obj.operation_type,
        'retention_duration_days': od.retention_duration_days,
        'max_backups': od.max_backups,
        'year_backups': od.year_backups,
        'day_backups': od.day_backups,
        'week_backups': od.week_backups,
        'month_backups': od.month_backups,
        'timezone': od.timezone,
        'start_time': obj.trigger.properties.start_time,
    }

    return data


def _add_vaults_to_policy_obj(obj, data, columns):
    """Add associated vaults to column and data tuples
    """
    i = 0
    for s in obj.associated_vaults:
        if obj.associated_vaults[i].vault_id:
            name = 'associated_vault_' + str(i + 1)
            data += (obj.associated_vaults[i].vault_id,)
            columns = columns + (name,)
            i += 1
    return data, columns


def _add_scheduling_patterns(obj, data, columns):
    """Add scheduling patterns to column and data tuples
    """
    i = 0
    for s in obj.trigger.properties.pattern:
        name = 'schedule_pattern_' + str(i + 1)
        data += (obj.trigger.properties.pattern[i],)
        columns = columns + (name,)
        i += 1
    return data, columns


class ListPolicies(command.Lister):
    _description = _('List CBR Policies')
    columns = ('ID', 'name', 'operation_type', 'start_time', 'enabled')

    def get_parser(self, prog_name):
        parser = super(ListPolicies, self).get_parser(prog_name)
        parser.add_argument(
            '--vault-id',
            metavar='<vault_id>',
            help=_('The ID of the vault.')
        )
        parser.add_argument(
            '--operation-type',
            metavar='<operation_type>',
            choices=['backup', 'replication'],
            help=_('Policy type')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        query = {}

        if parsed_args.vault_id:
            query['vault_id'] = parsed_args.vault_id
        if parsed_args.operation_type:
            query['operation_type'] = parsed_args.operation_type

        data = client.policies(**query)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_policy(s), self.columns,
                 ) for s in data))
        return table


class ShowPolicy(command.ShowOne):
    _description = _('Show single Policy details')
    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    def get_parser(self, prog_name):
        parser = super(ShowPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or name of the CBR policy.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        obj = client.find_policy(
            name_or_id=parsed_args.policy,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_policy(obj), self.columns)

        if obj.associated_vaults:
            data, self.columns = _add_vaults_to_policy_obj(
                obj, data, self.columns)

        if obj.trigger.properties.pattern:
            data, self.columns = _add_scheduling_patterns(
                obj, data, self.columns)

        return (self.columns, data)


class CreatePolicy(command.ShowOne):
    _description = _('Create CBR Policy')
    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    def get_parser(self, prog_name):
        parser = super(CreatePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the CBR Policy.')
        )
        parser.add_argument(
            '--disable',
            action='store_false',
            help=_('Disables CBR Policy which is enabled by default.')
        )
        parser.add_argument(
            '--operation-type',
            metavar='<operation_type>',
            default='backup',
            choices=['backup', 'replication'],
            help=_('backup or replication\n'
                   'Default: backup')
        )
        parser.add_argument(
            '--pattern',
            metavar='<pattern>',
            required=True,
            action='append',
            dest='patterns',
            help=_('Scheduling rule.\n'
                   'Repeat Option for multiple rules.')
        )
        parser.add_argument(
            '--day-backups',
            metavar='<day_backups>',
            type=int,
            help=_('Specifies the number of retained daily backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--week-backups',
            metavar='<week_backups>',
            type=int,
            help=_('Specifies the number of retained weekly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--month-backups',
            metavar='<month_backups>',
            type=int,
            help=_('Specifies the number of retained monthly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--year-backups',
            metavar='<year_backups>',
            type=int,
            help=_('Specifies the number of retained yearly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--timezone',
            metavar='<timezone>',
            help=_('Timezone where the user is located, e.g. UTC+08:00.')
        )
        parser.add_argument(
            '--max-backups',
            metavar='<max_backups>',
            type=int,
            help=_('Maximum number of retained backups.\n'
                   'Can be -1, backups will not cleared or ranges\n'
                   'from 0 to 99999.')
        )
        parser.add_argument(
            '--retention-duration-days',
            metavar='<retention_duration_days>',
            default=-1,
            type=int,
            help=_('Duration of retaining a backup, in days.\n'
                   '-1 indicates that the backups will not be '
                   'cleared based on the retention duration.\n'
                   'Ranges from 1 to 99999.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {
            'operation_definition': {},
            'trigger': {
                'properties': {
                    'pattern': []
                }
            }
        }

        # mandatory
        attrs['name'] = parsed_args.name
        attrs['enabled'] = parsed_args.disable
        attrs['operation_type'] = parsed_args.operation_type
        attrs['trigger']['properties']['pattern'] = parsed_args.patterns

        # optional
        if parsed_args.day_backups:
            attrs['operation_definition'].update(
                day_backups=parsed_args.day_backups)
        if parsed_args.week_backups:
            attrs['operation_definition'].update(
                week_backups=parsed_args.week_backups)
        if parsed_args.month_backups:
            attrs['operation_definition'].update(
                month_backups=parsed_args.month_backups)
        if parsed_args.year_backups:
            attrs['operation_definition'].update(
                year_backups=parsed_args.year_backups)
        if parsed_args.max_backups:
            attrs['operation_definition'].update(
                max_backups=parsed_args.max_backups)
        if parsed_args.retention_duration_days:
            rdd = parsed_args.retention_duration_days
            attrs['operation_definition'].update(
                retention_duration_days=rdd)

        if (parsed_args.day_backups
            or parsed_args.week_backups
            or parsed_args.month_backups
                or parsed_args.year_backups) and not parsed_args.timezone:
            msg = ("Parameter timezone must be provided if "
                   "<day|week|month|year>_backups are being used.")
            raise exceptions.BadRequest(msg)

        if parsed_args.timezone:
            attrs['operation_definition'].update(
                timezone=parsed_args.timezone)

        client = self.app.client_manager.cbr
        obj = client.create_policy(**attrs)

        data = utils.get_dict_properties(
            _flatten_policy(obj), self.columns)

        if obj.trigger.properties.pattern:
            data, self.columns = _add_scheduling_patterns(
                obj, data, self.columns)

        return (self.columns, data)


class UpdatePolicy(command.ShowOne):
    _description = _('Update CBR Policy')
    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    def get_parser(self, prog_name):
        parser = super(UpdatePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or name of the CBR Policy.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the CBR Policy.')
        )
        parser.add_argument(
            '--disable',
            action='store_true',
            help=_('Disables CBR Policy which is enabled by default.')
        )
        parser.add_argument(
            '--enable',
            action='store_true',
            help=_('Enables CBR Policy.')
        )
        parser.add_argument(
            '--pattern',
            metavar='<pattern>',
            action='append',
            dest='patterns',
            help=_('Scheduling rule.\n'
                   'Repeat Option for multiple rules.')
        )
        parser.add_argument(
            '--day-backups',
            metavar='<day_backups>',
            type=int,
            help=_('Specifies the number of retained daily backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--week-backups',
            metavar='<week_backups>',
            type=int,
            help=_('Specifies the number of retained weekly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--month-backups',
            metavar='<month_backups>',
            type=int,
            help=_('Specifies the number of retained monthly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--year-backups',
            metavar='<year_backups>',
            type=int,
            help=_('Specifies the number of retained yearly backups.\n'
                   'Ranges from 0 to 100.\n'
                   'If this parameter is configured, timezone is '
                   'mandatory.')
        )
        parser.add_argument(
            '--timezone',
            metavar='<timezone>',
            help=_('Timezone where the user is located, e.g. UTC+08:00.')
        )
        parser.add_argument(
            '--max-backups',
            metavar='<max_backups>',
            type=int,
            help=_('Maximum number of retained backups.\n'
                   'Can be -1, backups will not cleared or ranges\n'
                   'from 0 to 99999.')
        )
        parser.add_argument(
            '--retention-duration-days',
            metavar='<retention_duration_days>',
            type=int,
            help=_('Duration of retaining a backup, in days.\n'
                   '-1 indicates that the backups will not be '
                   'cleared based on the retention duration.\n'
                   'Ranges from 1 to 99999.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.enable:
            attrs['enabled'] = True
        if parsed_args.disable:
            attrs['enabled'] = False
        if parsed_args.patterns:
            trigger = {
                'trigger': {
                    'properties': {
                        'pattern': parsed_args.patterns
                    }
                }
            }
            attrs.update(trigger)

        if (parsed_args.day_backups
            or parsed_args.week_backups
            or parsed_args.month_backups
            or parsed_args.year_backups
            or parsed_args.max_backups
            or parsed_args.retention_duration_days
                or parsed_args.timezone):
            attrs['operation_definition'] = {}
            if parsed_args.day_backups:
                attrs['operation_definition'].update(
                    day_backups=parsed_args.day_backups)
            if parsed_args.week_backups:
                attrs['operation_definition'].update(
                    week_backups=parsed_args.week_backups)
            if parsed_args.month_backups:
                attrs['operation_definition'].update(
                    month_backups=parsed_args.month_backups)
            if parsed_args.year_backups:
                attrs['operation_definition'].update(
                    year_backups=parsed_args.year_backups)
            if parsed_args.max_backups:
                attrs['operation_definition'].update(
                    max_backups=parsed_args.max_backups)
            if parsed_args.retention_duration_days:
                rdd = parsed_args.retention_duration_days
                attrs['operation_definition'].update(
                    retention_duration_days=rdd)
            if parsed_args.timezone:
                attrs['operation_definition'].update(
                    timezone=parsed_args.timezone)

        client = self.app.client_manager.cbr
        policy = client.find_policy(
            name_or_id=parsed_args.policy,
            ignore_missing=False
        )

        if attrs:
            obj = client.update_policy(policy=policy.id, **attrs)
        else:
            obj = policy

        data = utils.get_dict_properties(
            _flatten_policy(obj), self.columns)

        if obj.associated_vaults:
            data, self.columns = _add_vaults_to_policy_obj(
                obj, data, self.columns)

        if obj.trigger.properties.pattern:
            data, self.columns = _add_scheduling_patterns(
                obj, data, self.columns)

        return (self.columns, data)


class DeletePolicy(command.Command):
    _description = _('Delete CBR Policy')

    def get_parser(self, prog_name):
        parser = super(DeletePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or name of the CBR Policy.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        policy = client.find_policy(
            name_or_id=parsed_args.policy,
            ignore_missing=False
        )

        self.app.client_manager.cbr.delete_policy(
            policy=policy.id,
            ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.dis.v2 import stream


EXAMPLE = {
    "stream_id": "8QM3Nt9YTLOwtUVYJhO",
    "stream_name": "newstream",
    "create_time": 1593569685875,
    "last_modified_time": 1599050091026,
    "retention_period": 24,
    "status": "RUNNING",
    "stream_type": "COMMON",
    "data_type": "BLOB",
    "writable_partition_count": 1,
    "readable_partition_count": 1,
    "tags": [],
    "auto_scale_enabled": False,
    "auto_scale_min_partition_count": 0,
    "auto_scale_max_partition_count": 0,
    "partitions": [
        {
            "status": "ACTIVE",
            "partition_id": "shardId-0000000000",
            "hash_range": "[0 : 9223372036854775807]",
            "sequence_number_range": "[289911 : 289927]"
        }
    ],
    "has_more_partitions": False
}


class TestStream(base.TestCase):

    def test_basic(self):
        sot = stream.Stream()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('stream_info_list', sot.resources_key)
        path = '/streams'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = stream.Stream(**EXAMPLE)
        updated_sot_attrs = (
            'auto_scale_enabled',
            'create_time',
            'last_modified_time',
            'stream_name',
        )
        self.assertEqual(EXAMPLE['create_time'], sot.created_at)
        self.assertEqual(EXAMPLE['stream_id'], sot.id)
        self.assertEqual(EXAMPLE['stream_name'], sot.name)
        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestQuota(TestApiG):
    app = None

    def setUp(self):
        super(TestQuota, self).setUp()
        self.create_gateway()
        self.gateway_id = TestQuota.gateway.id
        self.attrs = {
            "name": "app_demo",
            "remark": "Demo app"
        }
        TestQuota.app = self.client.create_app(self.gateway_id, **self.attrs)
        self.assertIsNotNone(self.app.id)
        self.addCleanup(
            self.client.delete_app,
            gateway=self.gateway_id,
            app=TestQuota.app.id,
        )

    def test_quotas(self):
        found = self.client.quotas(gateway=TestQuota.gateway.id,
                                   app=TestQuota.app.id,)
        self.assertIsNotNone(found)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import appcode


EXAMPLE_APP_CODE = {
    'gateway_id': 'gateway-67890',
    'app_id': 'app-12345',
    'app_code': 'code-12345',
    'id': 'code-1',
    'create_time': '2025-02-07T12:45:00Z'
}


class TestAppCode(base.TestCase):

    def test_basic(self):
        sot = appcode.AppCode()
        self.assertEqual('/apigw/instances/%(gateway_id)s/apps/'
                         '%(app_id)s/app-codes',
                         sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = appcode.AppCode(**EXAMPLE_APP_CODE)
        self.assertEqual(EXAMPLE_APP_CODE['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_APP_CODE['app_id'], sot.app_id)
        self.assertEqual(EXAMPLE_APP_CODE['app_code'], sot.app_code)
        self.assertEqual(EXAMPLE_APP_CODE['id'], sot.id)
        self.assertEqual(EXAMPLE_APP_CODE['create_time'], sot.create_time)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ExtraDHCPOpt(resource.Resource):
    #: Specifies the NTP server address configured for the subnet.
    opt_value = resource.Body('opt_value')
    #: Specifies the NTP server address name configured for the subnet.
    #: Currently, the value can only be set to ntp.
    opt_name = resource.Body('opt_name', default='ntp')


class Subnet(resource.Resource):
    resource_key = 'subnet'
    resources_key = 'subnets'
    base_path = '/v1/%(project_id)s/subnets'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters('vpc_id')

    # Properties
    project_id = resource.URI('project_id')
    description = resource.Body('description')
    #: Specifies the subnet CIDR block.
    cidr = resource.Body('cidr')
    #: Specifies the gateway of the subnet.
    gateway_ip = resource.Body('gateway_ip')
    #: Specifies whether DHCP is enabled for the subnet.
    dhcp_enable = resource.Body('dhcp_enable')
    #: Specifies the IP address of DNS server 1 on the subnet.
    primary_dns = resource.Body('primary_dns')
    #: Specifies the IP address of DNS server 2 on the subnet.
    secondary_dns = resource.Body('secondary_dns')
    #: Specifies the DNS server address list of a subnet.
    #: This field is required if use more than two DNS servers.
    dns_list = resource.Body('dnsList', type=list, list_type=str)
    #: Specifies the AZ to which the subnet belongs
    availability_zone = resource.Body('availability_zone')
    #: Specifies the ID of the VPC to which the subnet belongs.
    vpc_id = resource.Body('vpc_id')
    #: Specifies the NTP server address configured for the subnet.
    extra_dhcp_opts = resource.Body('extra_dhcp_opts', type=list,
                                    list_type=ExtraDHCPOpt)

    status = resource.Body('status')
    neutron_network_id = resource.Body('neutron_network_id')
    neutron_subnet_id = resource.Body('neutron_subnet_id')


def vpc_subnet_base_path(vpc_id):
    """Special case of subnet resource path"""
    return f'/v1/%(project_id)s/vpcs/{vpc_id}/subnets'



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Create MRS Job with attributes
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    "name": "my-mapreduce-job",
    "mains": [],
    "libs": [
        "092b628b-26a3-4571-9ba4-f8d000df8877"
    ],
    "is_protected": False,
    "interface": [],
    "is_public": False,
    "type": "MapReduce",
    "description": "This is the Map Reduce job template"
}

job = conn.mrs.create_job(**attrs)
print(job)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Member(resource.Resource):
    resource_key = 'member'
    resources_key = 'members'
    base_path = '/elb/pools/%(pool_id)s/members'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'address', 'admin_state_up', 'enterprise_project_id',
        'id', 'ip_version', 'name', 'protocol_port',
        'operating_status', 'subnet_cidr_id', 'weight',
        is_admin_state_up='admin_state_up'
    )

    # Properties
    #: Specifies the ID of the backend server group.
    pool_id = resource.URI('pool_id')

    #: Specifies the IP address of the backend server.
    address = resource.Body('address')
    #: Address parameter
    ip_version = resource.Body('ip_version')
    #: Specifies the administrative status of the backend server.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the operating status of the backend server.
    operating_status = resource.Body('operating_status')
    #: Specifies the project ID.
    project_id = resource.Body('project_id')
    #: Specifies the port used by the backend server to receive requests.
    protocol_port = resource.Body('protocol_port', type=int)
    #: Specifies the ID of the subnet where the backend server works.
    subnet_cidr_id = resource.Body('subnet_cidr_id')
    #: Specifies the weight of the backend server.
    weight = resource.Body('weight', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# import mock

from otcextensions.sdk.anti_ddos.v1 import _proxy
from otcextensions.sdk.anti_ddos.v1 import config as _config
from otcextensions.sdk.anti_ddos.v1 import floating_ip as _floating_ip
from otcextensions.sdk.anti_ddos.v1 import status as _status

from openstack.tests.unit import test_proxy_base


class TestAntiDdosProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestAntiDdosProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_floating_ips(self):
        self.verify_list(
            self.proxy.floating_ips, _floating_ip.FloatingIP,
            expected_kwargs={'paginated': False}
        )

    def test_unprotect_floating_ip(self):
        self.verify_delete(
            self.proxy.unprotect_floating_ip, _floating_ip.FloatingIP, True,
        )

    def test_get_floating_ip_policies(self):
        self.verify_get(
            self.proxy.get_floating_ip_policies, _floating_ip.FloatingIP,
        )

    def test_update_floating_ip_policies(self):
        self.verify_update(
            self.proxy.update_floating_ip_policies, _floating_ip.FloatingIP,
        )

    def test_configs(self):
        self.verify_list(
            self.proxy.configs, _config.Config,
            expected_kwargs={'paginated': False}
        )

    def test_get_floating_ip_status(self):
        self.verify_get(
            self.proxy.get_floating_ip_status, _status.FloatingIPStatus,
            expected_kwargs={
                'requires_id': False,
            }
        )

    def test_float_ip_logs(self):
        self.verify_list(
            self.proxy.floating_ip_events, _status.FloatingIPEvent,
            method_kwargs={
                'floating_ip_id': 'ip_id'
            },
            expected_kwargs={
                'paginated': False,
                'floating_ip_id': 'ip_id'
            }
        )

    def test_float_ip_stat_day(self):
        self.verify_list(
            self.proxy.floating_ip_stat_day, _status.FloatingIPDayStat,
            method_kwargs={
                'floating_ip_id': 'ip_id'
            },
            expected_kwargs={
                'paginated': False,
                'floating_ip_id': 'ip_id'
            }
        )

    def test_float_ip_stat_week(self):
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.floating_ip_stat_week,
            method_args=None,
            method_kwargs={
                'a': 'b'
            },
            expected_args=[_status.FloatingIPWeekStat],
            expected_kwargs={
                'a': 'b',
                'requires_id': False,
                'value': None
            }
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DatapointsSpec(resource.Resource):

    # Properties
    # time of the metric collection
    timestamp = resource.Body('timestamp')
    # Indicates the event type e.g. instance_host_info
    typespec = resource.Body('type')
    # host configuration information
    value = resource.Body('value')


class EventData(resource.Resource):

    base_path = '/event-data'

    # capabilities
    allow_list = True
    # allow_create = True

    _query_mapping = resource.QueryParameters(
        'dim.0', 'from', 'to', 'namespace', 'type'
    )

    # Properties
    # Datapoints retrieve the metrics data list
    datapoints = resource.Body('datapoints', type=list,
                               list_type=DatapointsSpec)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class MetricData(resource.Resource):
    base_path = '/apigw/instances/%(gateway_id)s/metric-data'
    resources_key = 'datapoints'
    _query_mapping = resource.QueryParameters('dim', 'metric_name', 'from',
                                              'to', 'period', 'filter')
    allow_list = True

    gateway_id = resource.URI('gateway_id')
    average = resource.Body('average', type=int)
    max = resource.Body('max', type=int)
    min = resource.Body('min', type=int)
    sum = resource.Body('sum', type=int)
    variance = resource.Body('variance', type=int)
    timestamp = resource.Body('timestamp')
    unit = resource.Body('unit')

    def get_metric_data(self, session, gateway_id, **params):
        """Retrieve metric data with the given parameters."""
        uri = f'apigw/instances/{gateway_id}/metric-data'
        response = session.get(uri, params=params)
        exceptions.raise_from_response(response)
        data = response.json()
        resources = data[self.resources_key]
        for raw_resource in resources:
            value = self.existing(
                connection=session._get_connection(),
                **raw_resource)
            yield value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Listener(resource.Resource):
    resource_key = 'listener'
    resources_key = 'listeners'
    base_path = '/elb/listeners'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'protocol_port', 'protocol', 'description',
        'default_tls_container_ref', 'client_ca_tls_container_ref',
        'connection_limit', 'default_pool_id',
        'id', 'name', 'http2_enable', 'loadbalancer_id', 'tls_ciphers_policy',
        'sni_container_refs', 'admin_state_up', 'tls_container_id'
    )

    # Properties
    #: Specifies the ID of the CA certificate used by the listener.
    client_ca_tls_container_ref = resource.Body('client_ca_tls_container_ref')
    #: Specifies the timeout duration for waiting
    #: for a request from a client, in seconds.
    client_timeout = resource.Body('client_timeout', type=int)
    #: Timestamp when the load balancer was created
    created_at = resource.Body('created_at')
    #: Specifies the maximum number of connections.
    connection_limit = resource.Body('connection_limit', type=int)
    #: The listener description
    description = resource.Body('description')
    #: Specifies the ID of the default backend server group.
    default_pool_id = resource.Body('default_pool_id')
    #: Specifies the ID of the server certificate used by the listener.
    default_tls_container_ref = resource.Body('default_tls_container_ref')
    #: Specifies whether to enable health check retries for backend servers.
    enable_member_retry = resource.Body('enable_member_retry', type=bool)
    #: Specifies whether to enable advanced forwarding.
    enhance_l7policy = resource.Body('enhance_l7policy_enable', type=bool)
    #: Specifies whether to use HTTP/2.
    http2_enable = resource.Body('http2_enable', type=bool)
    #: Dictionary of additional headers insertion into HTTP header.
    insert_headers = resource.Body('insert_headers', type=dict)
    #: Specifies the administrative status of the listener.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: List of load balancers associated with this listener.
    #: *Type: list of dicts which contain the load balancer IDs*
    load_balancers = resource.Body('loadbalancers', type=list, list_type=dict)
    #: Specifies the ID of the load balancer that the listener is added to.
    loadbalancer_id = resource.Body('loadbalancer_id')
    #: Specifies the IP address group associated with the listener.
    ipgroup = resource.Body('ipgroup', type=dict)
    #: Name of the listener.
    name = resource.Body('name')
    #: Specifies whether to enable health check retries for backend servers.
    keepalive_timeout = resource.Body('keepalive_timeout', type=int)
    #: Specifies the timeout duration for waiting
    #: for a request from a backend server, in seconds.
    member_timeout = resource.Body('member_timeout', type=int)
    #: The protocol of the listener, which is TCP, HTTP, HTTPS
    #: or TERMINATED_HTTPS.
    protocol = resource.Body('protocol')
    #: Port the listener will listen to, e.g. 80.
    protocol_port = resource.Body('protocol_port', type=int)
    #: The ID of the project this listener is associated with.
    project_id = resource.Body('project_id')
    #: Specifies the ID of the custom security policy.
    security_policy_id = resource.Body('security_policy_id')
    #: A list of references to TLS secrets.
    #: *Type: list*
    sni_container_refs = resource.Body('sni_container_refs', type=list)
    #: Specifies how wildcard domain name matches with the
    # SNI certificates used by the listener.
    sni_match_algo = resource.Body('sni_match_algo')
    #: Lists the tags.
    tags = resource.Body('tags', type=list)
    #: Specifies whether to pass source IP addresses of the clients to
    # backend servers.
    transparent_client_ip_enable = resource.Body(
        'transparent_client_ip_enable', type=bool)
    #: Specifies the security policy that will be used by the listener.
    tls_ciphers_policy = resource.Body('tls_ciphers_policy')
    #: Timestamp when the listener was last updated.
    updated_at = resource.Body('updated_at')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.dcs.v1 import backup
from otcextensions.tests.unit.osclient.dcs.v1 import fakes


class TestListBackup(fakes.TestDCS):

    objects = fakes.FakeBackup.create_multiple(3)
    inst = fakes.FakeInstance.create_one()

    columns = ('id', 'name', 'progress', 'status', 'error_code')

    data = []

    for s in objects:
        data.append((
            s.id,
            s.name,
            s.progress,
            s.status,
            s.error_code,
        ))

    def setUp(self):
        super(TestListBackup, self).setUp()

        self.cmd = backup.ListBackup(self.app, None)

        self.client.backups = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_list(self):
        arglist = [
            'inst'
        ]

        verifylist = [
            ('instance', 'inst')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backups.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backups.assert_called_once_with(
            instance={'id': self.inst.id},
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_query(self):
        arglist = [
            'inst',
            '--limit', '1',
            '--start', '2',
            '--from_time', '3',
            '--to_time', '4',
        ]

        verifylist = [
            ('instance', 'inst'),
            ('limit', 1),
            ('start', 2),
            ('from_time', '3'),
            ('to_time', '4'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backups.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backups.assert_called_once_with(
            instance={'id': self.inst.id},
            limit=1,
            start=2,
            begin_time='3',
            end_time='4'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteBackup(fakes.TestDCS):

    inst = fakes.FakeInstance.create_one()

    def setUp(self):
        super(TestDeleteBackup, self).setUp()

        self.cmd = backup.DeleteBackup(self.app, None)

        self.client.delete_instance_backup = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            'inst',
            't1',
            't2',
        ]
        verifylist = [
            ('instance', 'inst'),
            ('backup', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance_backup.side_effect = [{}, {}]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(instance_id=self.inst.id, backup='t1'),
            mock.call(instance_id=self.inst.id, backup='t2')
        ]

        self.client.delete_instance_backup.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_instance_backup.call_count)


class TestCreateBackup(fakes.TestDCS):

    _data = fakes.FakeBackup.create_one()
    inst = fakes.FakeInstance.create_one()

    columns = (
        'created_at', 'description', 'error_code', 'id',
        'name', 'period', 'progress', 'size', 'type', 'updated_at')

    data = (
        _data.created_at,
        _data.description,
        _data.error_code,
        _data.id,
        _data.name,
        _data.period,
        _data.progress,
        _data.size,
        _data.type,
        _data.updated_at
    )

    def setUp(self):
        super(TestCreateBackup, self).setUp()

        self.cmd = backup.CreateBackup(self.app, None)

        self.client.backup_instance = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_backup_create(self):
        arglist = [
            'inst',
            '--description', '2',
        ]
        verifylist = [
            ('instance', 'inst'),
            ('description', '2'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backup_instance.side_effect = [
            self._data
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backup_instance.assert_called_with(
            instance={'id': self.inst.id},
            description='2',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.cbr.v3 import _proxy
from otcextensions.sdk.cbr.v3 import backup as _backup
from otcextensions.sdk.cbr.v3 import checkpoint as _checkpoint
from otcextensions.sdk.cbr.v3 import member as _member
from otcextensions.sdk.cbr.v3 import policy as _policy
from otcextensions.sdk.cbr.v3 import restore as _restore
from otcextensions.sdk.cbr.v3 import task as _task
from otcextensions.sdk.cbr.v3 import vault as _vault


class TestCBRProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestCBRProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestCBRBackup(TestCBRProxy):

    def test_backups(self):
        self.verify_list(self.proxy.backups, _backup.Backup)

    def test_backup_delete(self):
        self.verify_delete(self.proxy.delete_backup,
                           _backup.Backup, True)

    def test_backup_find(self):
        self.verify_find(self.proxy.find_backup, _backup.Backup)

    def test_backup_get(self):
        self.verify_get(self.proxy.get_backup, _backup.Backup)

    def test_member_add(self):
        members = ['member1']
        backup = _backup.Backup(id='backup')
        m1 = _member.Member(id='member1')
        self.proxy._get = mock.Mock()
        self.proxy._get.return_value = m1
        self._verify(
            'otcextensions.sdk.cbr.v3.backup.Backup.add_members',
            self.proxy.add_members,
            method_args=[backup, members],
            expected_args=[self.proxy],
            expected_result=[m1],
            expected_kwargs={'members': members}
        )


class TestCBRPolicy(TestCBRProxy):

    def test_policies(self):
        self.verify_list(self.proxy.policies, _policy.Policy)

    def test_policy_find(self):
        self.verify_find(self.proxy.find_policy, _policy.Policy)

    def test_policy_get(self):
        self.verify_get(self.proxy.get_policy, _policy.Policy)

    def test_policy_create(self):
        self.verify_create(self.proxy.create_policy,
                           _policy.Policy,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_policy_delete(self):
        self.verify_delete(self.proxy.delete_policy,
                           _policy.Policy, True)


class TestCBRCheckpoint(TestCBRProxy):

    def test_checkpoint_get(self):
        self.verify_get(self.proxy.get_checkpoint, _checkpoint.Checkpoint)

    def test_checkpoint_create(self):
        self.verify_create(self.proxy.create_checkpoint,
                           _checkpoint.Checkpoint,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})


class TestCBRRestore(TestCBRProxy):

    def test_restore_server(self):
        attrs = {'a': 'b'}
        backup = _backup.Backup(id='my_backup')
        self._verify(
            'openstack.proxy.Proxy._create',
            self.proxy.restore_data,
            method_args=[backup],
            method_kwargs=attrs,
            expected_args=[_restore.Restore],
            expected_kwargs=dict(
                backup_id=backup.id,
                **attrs
            )
        )


class TestCBRVault(TestCBRProxy):

    def test_vaults(self):
        self.verify_list(self.proxy.vaults, _vault.Vault)

    def test_vault_find(self):
        self.verify_find(self.proxy.find_vault, _vault.Vault)

    def test_vault_get(self):
        self.verify_get(self.proxy.get_vault, _vault.Vault)

    def test_vault_create(self):
        self.verify_create(self.proxy.create_vault,
                           _vault.Vault,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_vault_delete(self):
        self.verify_delete(self.proxy.delete_vault,
                           _vault.Vault, True)

    def test_bind_policy(self):
        vault = _vault.Vault(id='vault')
        policy = _policy.Policy(id='policy')
        self._verify(
            'otcextensions.sdk.cbr.v3.vault.Vault.bind_policy',
            self.proxy.bind_policy,
            method_args=[vault, policy],
            expected_args=[self.proxy],
            expected_kwargs={'policy_id': 'policy'}
        )

    def test_unbind_policy(self):
        vault = _vault.Vault(id='vault')
        policy = _policy.Policy(id='policy')
        self._verify(
            'otcextensions.sdk.cbr.v3.vault.Vault.unbind_policy',
            self.proxy.unbind_policy,
            method_args=[vault, policy],
            expected_args=[self.proxy],
            expected_kwargs={'policy_id': 'policy'}
        )

    def test_associate_resources(self):
        vault = _vault.Vault(id='vault')
        resources = 'resources'
        self._verify(
            'otcextensions.sdk.cbr.v3.vault.Vault.associate_resources',
            self.proxy.associate_resources,
            method_args=[vault, resources],
            expected_args=[self.proxy, resources],
        )

    def test_dissociate_resources(self):
        vault = _vault.Vault(id='vault')
        resources = 'resources'
        self._verify(
            'otcextensions.sdk.cbr.v3.vault.Vault.dissociate_resources',
            self.proxy.dissociate_resources,
            method_args=[vault, resources],
            expected_args=[self.proxy, resources],
        )


class TestCBRMember(TestCBRProxy):

    def test_list(self):
        backup = _backup.Backup(id='backup')
        self.verify_list(
            self.proxy.members,
            _member.Member,
            method_args=[backup],
            expected_kwargs={'backup_id': backup.id},
            expected_args=[]
        )

    def test_get(self):
        backup = _backup.Backup(id='backup')
        member = _member.Member(id='member')
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_member,
            method_args=[backup, member],
            method_kwargs={},
            expected_args=[_member.Member, member],
            expected_kwargs={'backup_id': backup.id}
        )

    def test_delete(self):
        backup = _backup.Backup(id='backup')
        member = _member.Member(id='member')
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_member,
            method_args=[backup, member, True],
            method_kwargs={},
            expected_args=[_member.Member, member],
            expected_kwargs={
                'backup_id': backup.id,
                'ignore_missing': True}
        )

    def test_update(self):
        backup = _backup.Backup(id='backup')
        member = _member.Member(id='member')
        status = 'accepted'
        vault = _vault.Vault(id='vault')
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_member,
            method_args=[backup, member, status, vault],
            method_kwargs={},
            expected_args=[_member.Member, member],
            expected_kwargs={
                'backup_id': backup.id,
                'status': status,
                'vault_id': vault.id}
        )


class TestCBRTask(TestCBRProxy):

    def test_tasks(self):
        self.verify_list(self.proxy.tasks, _task.Task)

    def test_task_get(self):
        self.verify_get(self.proxy.get_task, _task.Task)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class VirtualInterface(resource.Resource):

    resource_key = 'virtual_interface'
    resources_key = 'virtual_interfaces'
    base_path = '/dcaas/virtual-interfaces'

    # capabilities
    allow_list = True
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'id', 'project_id', 'name', 'description', 'direct_connect_id',
        'vgw_id', 'type', 'service_type', 'vlan', 'bandwidth',
        'local_gateway_v4_ip', 'remote_gateway_v4_ip', 'route_mode', 'bgp_asn',
        'bgp_md5', 'remote_ep_group_id', 'service_ep_group_id',
        'admin_state_up', 'create_time', 'delete_time', 'status', 'rate_limit',
        project_id='tenant_id')

    # Properties
    #: ID of the direct connection.
    id = resource.Body('id')
    #: Indicates the virtual interface name.
    name = resource.Body('name')
    #: Project id.
    project_id = resource.Body('tenant_id')
    #: Indicates the description of the virtual interface.
    description = resource.Body('description')
    #: Specifies the connection ID.
    direct_connect_id = resource.Body('direct_connect_id')
    #: Specifies the virtual gateway ID.
    vgw_id = resource.Body('vgw_id')
    #: Specifies the virtual interface type. The value can be public
    # or private.
    type = resource.Body('type')
    #: Specifies the access service type. The value can be vpc, public service,
    # or vpc and public service.
    service_type = resource.Body('service_type')
    #: Specifies the VLAN used by the local gateway to communicate with
    # the remote gateway.
    vlan = resource.Body('vlan', type=int)
    #: Specifies the virtual interface bandwidth.
    bandwidth = resource.Body('bandwidth', type=int)
    #: Specifies the IPv4 address of the local gateway.
    local_gateway_v4_ip = resource.Body('local_gateway_v4_ip')
    #: Specifies the IPv4 address of the remote gateway.
    remote_gateway_v4_ip = resource.Body('remote_gateway_v4_ip')
    #: Specifies the routing mode. The value can be static or bgp.
    route_mode = resource.Body('route_mode')
    #: Specifies the AS number of the BGP peer.
    bgp_asn = resource.Body('bgp_asn', type=int)
    #: Specifies the MD5 password of the BGP peer.
    bgp_md5 = resource.Body('bgp_md5')
    #: Specifies the ID of the remote endpoint group that records the tenant
    # CIDR blocks.
    remote_ep_group_id = resource.Body('remote_ep_group_id')
    #: Specifies the ID of the service endpoint group that records the public
    # service CIDR blocks.
    service_ep_group_id = resource.Body('service_ep_group_id')
    #: Specifies the time when the virtual interface is created.
    create_time = resource.Body('create_time')
    #: Specifies the time when the virtual interface is deleted.
    delete_time = resource.Body('delete_time')
    #: Specifies the administrative status of the virtual interface.\
    # The value can be true or false.
    admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies whether to limit the traffic rate. The value can be true
    # or false.
    rate_limit = resource.Body('rate_limit', type=bool)
    #: Specifies the virtual interface status. The value can be ACTIVE, DOWN,
    # BUILD, ERROR, PENDING_CREATE, PENDING_UPDATE, or PENDING_DELETE.
    status = resource.Body('status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class HealthMonitor(resource.Resource):
    resource_key = 'healthmonitor'
    resources_key = 'healthmonitors'
    base_path = '/elb/healthmonitors'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'delay', 'domain_name',
        'expected_codes', 'id', 'http_method',
        'max_retries', 'max_retries_down', 'monitor_port',
        'name', 'timeout', 'type', 'url_path',
        is_admin_state_up='admin_state_up',
    )

    # Properties
    #: Specifies the interval between health checks, in seconds.
    delay = resource.Body('delay', type=int)
    #: Specifies the domain name that HTTP requests are
    #: sent to during the health check.
    domain_name = resource.Body('domain_name')
    #: Specifies the expected HTTP status code.
    expected_codes = resource.Body('expected_codes')
    #: Specifies the HTTP method.
    http_method = resource.Body('http_method')
    #: Specifies the administrative status of the health check.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the number of consecutive health checks when
    #: the health check result of a backend server changes from
    #: OFFLINE to ONLINE.
    max_retries = resource.Body('max_retries', type=int)
    #: Specifies the number of consecutive health checks when
    #: the health check result of a backend server changes from
    #: ONLINE to OFFLINE
    max_retries_down = resource.Body('max_retries_down', type=int)
    #: Specifies the port used for the health check.
    monitor_port = resource.Body('monitor_port', type=int)
    #: Specifies the ID of the backend server group for which
    #: the health check is configured.
    pool_id = resource.Body('pool_id')
    #: Lists the IDs of backend server groups for which
    #: the health check is configured.
    pools = resource.Body('pools', type=list)
    #: Specifies the project ID.
    project_id = resource.Body('project_id')
    #: Specifies the maximum time required for waiting for a response
    #: from the health check, in seconds.
    timeout = resource.Body('timeout', type=int)
    #: Specifies the health check protocol.
    type = resource.Body('type')
    #: Specifies the HTTP request path for the health check.
    url_path = resource.Body('url_path')
    #: Specifies the time when the health check was configured.
    created_at = resource.Body('created_at')
    #: Specifies the time when the health check was updated.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.dis.v2 import dump_task

EXAMPLE = {
    "create_time": 1689377676849,
    "destination_type": "OBS",
    "exception_strategy": "ignoreAndBackup",
    "last_transfer_timestamp": 1689377695587,
    "obs_destination_description": {
        "agency_name": "dis_admin_agency",
        "consumer_strategy": "LATEST",
        "deliver_time_interval": 300,
        "destination_file_type": "text",
        "obs_bucket_path": "test-bucket-abcdef",
        "record_delimiter": "\n",
        "retry_duration": 0
    },
    "partitions": [
        {
            "discard": 0,
            "last_transfer_offset": 0,
            "last_transfer_timestamp": 1689377695587,
            "partitionId": "shardId-0000000000",
            "state": "RUNNING"
        },
        {
            "discard": 0,
            "last_transfer_offset": 0,
            "last_transfer_timestamp": 1689377695587,
            "partitionId": "shardId-0000000001",
            "state": "RUNNING"
        }
    ],
    "state": "RUNNING",
    "streamId": "bxdSOIEYPwJdSlAqq5z",
    "stream_id": "bxdSOIEYPwJdSlAqq5z",
    "stream_name": "test-dis",
    "task_id": "lJ4Ts5ObPd2UOpcRj3K",
    "task_name": "test-dump-task"
}


class TestDumpTask(base.TestCase):

    def test_basic(self):
        sot = dump_task.DumpTask()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('tasks', sot.resources_key)
        path = '/streams/%(uri_stream_name)s/transfer-tasks'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_patch)

    def test_make_it(self):
        sot = dump_task.DumpTask(**EXAMPLE)
        updated_sot_attrs = (
            'create_time',
            'state',
            'streamId',
            'obs_destination_description',
        )
        self.assertEqual(EXAMPLE['create_time'], sot.created_at)
        self.assertEqual(EXAMPLE['state'], sot.status)
        self.assertEqual(EXAMPLE['streamId'], sot.stream_id)
        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)

        obs_dest = EXAMPLE['obs_destination_description']
        sot_obs_dest = dump_task.ObsDestinationSpec(**obs_dest)
        for key, value in obs_dest.items():
            self.assertEqual(getattr(sot_obs_dest, key), value)
